meta

package
v0.0.0-...-7a4d71e Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 8, 2026 License: AGPL-3.0 Imports: 22 Imported by: 0

Documentation

Overview

Package meta provides extract processors to retrieve several meta information from a page (meta tags, favicon, pictures...).

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func ExtractFavicon

func ExtractFavicon(m *extract.ProcessMessage, next extract.Processor) extract.Processor

ExtractFavicon is a processor that extracts the favicon for the first extracted document.

func ExtractMeta

ExtractMeta is a processor that extracts metadata from the document and set the Drop values accordingly.

func ExtractOembed

func ExtractOembed(m *extract.ProcessMessage, next extract.Processor) extract.Processor

ExtractOembed is a processor that extracts the picture from the document metadata. It has to come after ExtractMeta.

func ExtractPicture

func ExtractPicture(m *extract.ProcessMessage, next extract.Processor) extract.Processor

ExtractPicture is a processor that extracts the picture from the document metadata. It has to come after ExtractMeta.

func ParseMeta

func ParseMeta(doc *html.Node) extract.DropMeta

ParseMeta parses page metadata.

func ParseProps

func ParseProps(doc *html.Node) extract.DropProperties

ParseProps parses raw properties from the page. The results for link and meta are a bit redundant with what we fetch with ParseMeta but they contain more information and can be useful to content scripts.

func SetDropProperties

func SetDropProperties(m *extract.ProcessMessage, next extract.Processor) extract.Processor

SetDropProperties will set some Drop properties bases on the retrieved metadata. It must be run after ExtractMeta and ExtractOembed.

Types

This section is empty.

Directories

Path Synopsis
Package tinymeta implements reading metadata from HTML documents without having to parse the entire DOM in memory.
Package tinymeta implements reading metadata from HTML documents without having to parse the entire DOM in memory.

Jump to