Documentation
¶
Overview ¶
Package srcset is an srcset value parser.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type ImageSource ¶
ImageSource is a structure that contains an image definition.
func (ImageSource) String ¶
func (s ImageSource) String() string
type SourceSet ¶
type SourceSet []ImageSource
SourceSet is the result of parsing the value of a srcset attribute. A SourceSet consists of multiple ImageSource instances.
Click to show internal directories.
Click to hide internal directories.